@@ -29,6 +29,7 @@
 #include <QUrl>
 #include <QUrlQuery>
 #include <QPlainTextEdit>
+#include <QMessageBox>
 
 #include <QJsonArray>
 #include <QJsonDocument>
@@ -37,6 +38,8 @@
 
 
 #include <QStringList>
+#include <QSslCipher>
+
 
 #if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
 
@@ -58,6 +61,7 @@
 #include "panelgenerator.h"
 #include "wakeonlan.h"
 
+int MainWindow::isSessionOpen = false;
 int hexToInt(QChar hex);
 void parserMajorMinorBuild(QString sw, unsigned int &major, unsigned int &minor, unsigned int &build);
 extern void themeTheButton(QPushButton * button);
@@ -68,10 +72,60 @@ MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
     ui(new Ui::MainWindow)
 {
+
     ui->setupUi(this);
 
 
+    QCheckBox* leaveSessionOpen;
+    QCheckBox* twoVerify;
+    //QLineEdit* hostName = ui->hostName;
+
     QSettings settings(SETTINGSFILE, QSettings::IniFormat);
+    //leaveSessionOpen
+    if(settings.value("leaveSessionOpen").toString() == "false"){
+        ui->leaveSessionOpen->setChecked(false);
+    } else {
+        ui->leaveSessionOpen->setChecked(true);
+    }
+
+    leaveSessionOpen = ui->leaveSessionOpen;
+    connect(leaveSessionOpen, &QCheckBox::toggled, this, &MainWindow::on_leaveSessionOpen_StateChanged);
+
+    //twoVerify
+    if(settings.value("twoVerify").toString() == "false"){
+        ui->twoVerify->setChecked(false);
+    } else {
+        ui->twoVerify->setChecked(true);
+    }
+
+    twoVerify = ui->twoVerify;
+    connect(twoVerify, &QCheckBox::toggled, &packetNetwork , &PacketNetwork::on_twoVerify_StateChanged);
+
+    //hostName
+    connect(ui->hostName, &QLineEdit::editingFinished, this, &MainWindow::on_hostName_editingFinished);
+
+
+    //cipher comboBox
+    cipherCb = ui->cipherCb;
+    //add the combobox the correct cipher suites
+    QList<QSslCipher> ciphers = QSslConfiguration::supportedCiphers();
+    for (const QSslCipher &cipher : ciphers) {
+        cipherCb->addItem(cipher.name());
+    }
+
+    if(!PacketNetwork::DTLSisSupported()) {
+        int itemAt = ui->udptcpComboBox->findText("DTLS");
+        if(itemAt > 0) {
+            ui->udptcpComboBox->removeItem(itemAt);
+        }
+    }
+
+
+    if ( ui->udptcpComboBox->currentText().toLower() != "dtls"){
+        ui->dtlsFrame->hide();
+    }
+
+    connect(cipherCb, &QComboBox::editTextChanged, this, &MainWindow::on_cipherCb_currentIndexChanged);
 
     QIcon mIcon(":pslogo.png");
 
@@ -82,7 +136,7 @@ MainWindow::MainWindow(QWidget *parent) :
     setWindowIcon(mIcon);
 
     tableActive = false;
-    darkMode = settings.value("darkModeCheck", true).toBool();
+    darkMode = Settings::useDark();
 
     PanelGenerator::darkMode = darkMode;
 
@@ -159,6 +213,9 @@ MainWindow::MainWindow(QWidget *parent) :
              connect(&packetNetwork, SIGNAL(packetSent(Packet)),
                      this, SLOT(toTrafficLog(Packet)));
 
+             connect(&packetNetwork, SIGNAL(packetReceived(Packet)), this, SLOT(toTrafficLog(Packet)));
+
+
     if( !QFile::exists(PACKETSFILE)) {
         // Packets file does not exist. Load starter set.
         QFile starterfile(":/starter_set.json");
@@ -244,6 +301,24 @@ MainWindow::MainWindow(QWidget *parent) :
 
     stopResendingButton->hide();
 
+
+    dtlsServerStatus = new QPushButton("DTLS:" + packetNetwork.getDTLSPortString());
+    themeTheButton(dtlsServerStatus);
+    dtlsServerStatus->setIcon(QIcon(DTLSRXICON));
+
+    connect(dtlsServerStatus, SIGNAL(clicked()),
+            this, SLOT(toggleDTLSServer()));
+
+
+
+    if(PacketNetwork::DTLSisSupported()) {
+        statusBar()->insertPermanentWidget(2, dtlsServerStatus);
+    }
+
+
+
+
+
     udpServerStatus = new QPushButton("UDP:" + packetNetwork.getUDPPortString());
     themeTheButton(udpServerStatus);
     udpServerStatus->setIcon(QIcon(UDPRXICON));
@@ -252,7 +327,7 @@ MainWindow::MainWindow(QWidget *parent) :
             this, SLOT(toggleUDPServer()));
 
 
-    statusBar()->insertPermanentWidget(2, udpServerStatus);
+    statusBar()->insertPermanentWidget(3, udpServerStatus);
 
 
     //updatewidget
@@ -272,15 +347,17 @@ MainWindow::MainWindow(QWidget *parent) :
             this, SLOT(toggleSSLServer()));
 
 
-    statusBar()->insertPermanentWidget(3, tcpServerStatus);
+    statusBar()->insertPermanentWidget(4, tcpServerStatus);
+
+
+    statusBar()->insertPermanentWidget(5, sslServerStatus);
 
 
-    statusBar()->insertPermanentWidget(4, sslServerStatus);
 
     //ipmode toggle
     IPmodeButton = new QPushButton("IPv4 Mode");
     themeTheButton(IPmodeButton);
-    statusBar()->insertPermanentWidget(5, IPmodeButton);
+    statusBar()->insertPermanentWidget(6, IPmodeButton);
 
     setIPMode();
 
@@ -289,7 +366,7 @@ MainWindow::MainWindow(QWidget *parent) :
     connect(IPmodeButton, SIGNAL(clicked()),
             this, SLOT(toggleIPv4_IPv6()));
 
-
+    DTLSServerStatus();
     UDPServerStatus();
     TCPServerStatus();
     SSLServerStatus();
@@ -380,50 +457,6 @@ MainWindow::MainWindow(QWidget *parent) :
     }
 
 
-    //Now that the UI is loaded, create the settings folders if they do not exist
-    QDir mdir;
-    mdir.mkpath(TEMPPATH);
-    mdir.mkpath(SETTINGSPATH);
-
-
-
-    //this is stored as base64 so smart git repos
-    //do not complain about shipping a private key.
-    QFile snakeoilKey("://ps.key.base64");
-    QFile snakeoilCert("://ps.pem.base64");
-
-
-    QString defaultCertFile = CERTFILE;
-    QString defaultKeyFile = KEYFILE;
-
-    QFile certfile(defaultCertFile);
-    QFile keyfile(defaultKeyFile);
-    QByteArray decoded;
-    decoded.clear();
-
-    if (!certfile.exists()) {
-        if (snakeoilCert.open(QFile::ReadOnly)) {
-            decoded = QByteArray::fromBase64(snakeoilCert.readAll());
-            snakeoilCert.close();
-        }
-        if (certfile.open(QFile::WriteOnly)) {
-            certfile.write(decoded);
-            certfile.close();
-        }
-    }
-
-    if (!keyfile.exists()) {
-        if (snakeoilKey.open(QFile::ReadOnly)) {
-            decoded = QByteArray::fromBase64(snakeoilKey.readAll());
-            snakeoilKey.close();
-        }
-        if (keyfile.open(QFile::WriteOnly)) {
-            keyfile.write(decoded);
-            keyfile.close();
-        }
-    }
-
-
     updateManager(QByteArray());
 
     //on_actionExport_Packets_JSON_triggered();
@@ -479,6 +512,13 @@ MainWindow::MainWindow(QWidget *parent) :
 
 
 
+void MainWindow::toggleDTLSServer()
+{
+    QSettings settings(SETTINGSFILE, QSettings::IniFormat);
+    settings.setValue("dtlsServerEnable", !settings.value("dtlsServerEnable", true).toBool());
+    applyNetworkSettings();
+}
+
 void MainWindow::toggleUDPServer()
 {
     QSettings settings(SETTINGSFILE, QSettings::IniFormat);
@@ -754,6 +794,28 @@ void MainWindow::toTrafficLog(Packet logPacket)
 
 }
 
+void MainWindow::DTLSServerStatus()
+{
+
+    if (packetNetwork.DTLSListening()) {
+        QString ports = packetNetwork.getDTLSPortString();
+        int portcount = packetNetwork.getDTLSPortsBound().size();
+        dtlsServerStatus->setToolTip(ports);
+        if(portcount > 3) {
+            dtlsServerStatus->setText("DTLS: " + QString::number(portcount) + tr(" Ports"));
+        } else {
+            dtlsServerStatus->setText("DTLS:" + ports);
+        }
+
+    } else {
+        dtlsServerStatus->setText(tr("DTLS Server Disabled"));
+
+    }
+
+    //updatewidget
+
+
+}
 
 void MainWindow::UDPServerStatus()
 {
@@ -1389,6 +1451,17 @@ void MainWindow::on_deletePacketButton_clicked()
 void MainWindow::on_packetIPEdit_lostFocus()
 {
     QString ipPacket = ui->packetIPEdit->text().trimmed();
+
+    // Allow pasting URLs and then extract the host name.
+    if(ipPacket.startsWith("http://") || ipPacket.startsWith("https://") ) {
+        QUrl ipPacketURL(ipPacket);
+        if(ipPacketURL.isValid()) {
+            ipPacket = ipPacketURL.host();
+            ui->packetIPEdit->setText(ipPacket);
+        }
+    }
+
+
     QHostAddress address(ipPacket);
     if (QAbstractSocket::IPv4Protocol == address.protocol()) {
         QDEBUG() << "Valid IPv4 address.";
@@ -1933,6 +2006,7 @@ void MainWindow::applyNetworkSettings()
     ui->persistentTCPCheck->setChecked(settings.value("persistentTCPCheck", false).toBool());
     on_persistentTCPCheck_clicked(ui->persistentTCPCheck->isChecked());
 
+    DTLSServerStatus();
     UDPServerStatus();
     TCPServerStatus();
     SSLServerStatus();
@@ -2071,26 +2145,50 @@ void MainWindow::on_saveLogButton_clicked()
     static QString fileName = QDir::homePath() + QString("/trafficlog.log");
 
     fileName = QFileDialog::getSaveFileName(this, tr("Save Traffic Log"),
-                                            QDir::toNativeSeparators(fileName), tr("log (*.log)"));
+                                            QDir::toNativeSeparators(fileName), "Log (*.log);;CSV (*.csv)");
+
+    if(fileName.isEmpty()) {
+        return;
+    }
+
+    QStringList headers;
+    headers << tr("TIME")  << tr("From IP")  << tr("From Port")  << tr("To IP")
+         << tr("To Port")  << tr("Method")  << tr("Error")  << "ASCII"  << "Hex";
+
+
 
     QStringList testExt = fileName.split(".");
     QString ext = "";
+    QString delim = "\t";
+
+
+
     if (testExt.size() > 0) {
         ext = testExt.last();
     }
-    if (ext != "log") {
-        fileName.append(".log");
+
+    bool csv = ext.toLower().trimmed() == "csv";
+
+
+    if (csv) {
+        QDEBUG() << "Export csv";
+        delim = ",";
+    } else {
+        if (ext != "log") {
+            fileName.append(".log");
+        }
     }
+
+
+
     QDEBUG() << "Export log to" << fileName;
 
+
     QString exportString = "";
-    QString delim = "\t";
 
     QTextStream out;
     out.setString(&exportString);
-
-    out << tr("TIME") << delim << tr("From IP") << delim << tr("From Port") << delim << tr("To IP")
-        << delim << tr("To Port") << delim << tr("Method") << delim << tr("Error") << delim << "ASCII" << delim << "Hex\n";
+    out << headers.join(delim) << "\n";
 
 
     Packet tempPacket;
@@ -2110,7 +2208,17 @@ void MainWindow::on_saveLogButton_clicked()
         exportString.append(delim);
         exportString.append(tempPacket.errorString);
         exportString.append(delim);
-        exportString.append(tempPacket.hexToASCII(tempPacket.hexString));
+
+        QString escapeAscii = tempPacket.hexToASCII(tempPacket.hexString);
+        if(escapeAscii.contains(delim)) {
+            if(escapeAscii.contains("\"")) {
+                escapeAscii.replace("\"", "\"\"");
+            }
+            escapeAscii = "\"" + escapeAscii + "\"";
+        }
+
+
+        exportString.append(escapeAscii);
         exportString.append(delim);
         exportString.append(tempPacket.hexString);
         exportString.append(delim);
@@ -2242,9 +2350,37 @@ void MainWindow::on_actionHelp_triggered()
     QDesktopServices::openUrl(QUrl("https://packetsender.com/documentation"));
 }
 
+
+
+void MainWindow::on_leaveSessionOpen_StateChanged(){
+    //ui.checkBox->setChecked(checkBoxState);
+
+    QSettings settings(SETTINGSFILE, QSettings::IniFormat);
+    QString leaveSessionOpen = settings.value("leaveSessionOpen", "false").toString();
+    if(leaveSessionOpen == "false"){
+        settings.setValue("leaveSessionOpen", "true");
+    }
+    else{
+        settings.setValue("leaveSessionOpen", "false");
+    }
+}
+
+void MainWindow::on_sendSimpleAck_StateChanged(){
+    //ui.checkBox->setChecked(checkBoxState);
+
+    QSettings settings(SETTINGSFILE, QSettings::IniFormat);
+    QString sendSimpleAck = settings.value("sendSimpleAck", "false").toString();
+    if(sendSimpleAck == "false"){
+        settings.setValue("sendSimpleAck", "true");
+    }
+    else{
+        settings.setValue("sendSimpleAck", "false");
+    }
+}
+
 void MainWindow::on_actionSettings_triggered()
 {
-    Settings settings;
+    Settings settings(this);
     int accepted = settings.exec();
     if (accepted) {
         setIPMode();
@@ -2581,6 +2717,7 @@ void MainWindow::on_loadFileButton_clicked()
 
 }
 
+
 void MainWindow::on_actionDonate_Thank_You_triggered()
 {
 
@@ -2595,6 +2732,15 @@ void MainWindow::on_udptcpComboBox_currentIndexChanged(const QString &arg1)
     auto isHttp = selectedText.contains("http");
     auto isPost = selectedText.contains("post") && isHttp;
 
+    /////////////////////////////////dtls add line edit for adding path for cert
+
+    if ( ui->udptcpComboBox->currentText().toLower() == "dtls") {
+        ui->dtlsFrame->show();
+    } else {
+        ui->dtlsFrame->hide();
+    }
+
+
     if(isHttp) {
         ui->asciiLabel->setText("Data");
     } else {
@@ -2626,6 +2772,68 @@ void MainWindow::on_udptcpComboBox_currentIndexChanged(const QString &arg1)
     ui->genPostDataButton->setVisible(isPost);
 }
 
+
+
+void MainWindow::on_actionWake_On_LAN_Magic_Packet_triggered()
+{
+    static QString previousMAC = "";
+    static int previousPortIndex = 0;
+    WakeOnLAN * wol = new WakeOnLAN(this);
+    wol->setTarget(previousMAC, previousPortIndex);
+    wol->setModal(true);
+    wol->exec();
+    if(!wol->generatedPacket.errorString.isEmpty()) {
+
+        QMessageBox msgBox;
+        msgBox.setWindowIcon(QIcon(":pslogo.png"));
+        msgBox.setWindowTitle(tr("Invalid MAC Address."));
+        msgBox.setStandardButtons(QMessageBox::Ok);
+        msgBox.setDefaultButton(QMessageBox::Ok);
+        msgBox.setIcon(QMessageBox::Critical);
+        msgBox.setText(wol->generatedPacket.errorString);
+        msgBox.setModal(true);
+        msgBox.exec();
+
+        on_actionWake_On_LAN_Magic_Packet_triggered();
+        wol->deleteLater();
+        return;
+    }
+
+    if(wol->generatedPacket.toIP.isEmpty()) {
+        wol->deleteLater();
+        return;
+    }
+
+    previousMAC = wol->mac;
+    previousPortIndex = wol->portIndex;
+
+    ui->packetIPEdit->setText(wol->generatedPacket.toIP);
+    ui->packetPortEdit->setText(QString::number(wol->generatedPacket.port));
+    ui->resendEdit->setText("0");
+    int findtext = ui->udptcpComboBox->findText(wol->generatedPacket.tcpOrUdp);
+    if (findtext > -1) {
+        ui->udptcpComboBox->setCurrentIndex(findtext);
+    }
+    ui->packetHexEdit->setText(wol->generatedPacket.hexString);
+    on_packetHexEdit_editingFinished();
+    on_udptcpComboBox_currentIndexChanged("");
+
+    if(ui->packetNameEdit->text().isEmpty()) {
+        ui->packetNameEdit->setFocus();
+    }
+    wol->deleteLater();
+}
+
+
+void MainWindow::on_cipherCb_currentIndexChanged(){
+    QSettings settings(SETTINGSFILE, QSettings::IniFormat);
+    settings.setValue("cipher", cipherCb->currentText());
+    //create new session even if the leave open session checkbox is pushed create new session, because the cipher has been changed
+    isSessionOpen = false;
+
+}
+
+
 void MainWindow::on_genPostDataButton_clicked()
 {
     PostDataGen * phttp = new PostDataGen(this, ui->packetASCIIEdit->text());
@@ -2791,54 +2999,9 @@ void MainWindow::on_udptcpComboBox_currentIndexChanged(int index)
     on_udptcpComboBox_currentIndexChanged("");
 }
 
+void MainWindow::on_hostName_editingFinished(){
+    QSettings settings(SETTINGSFILE, QSettings::IniFormat);
+    settings.setValue("hostNameEdit", ui->hostName->text());
 
-void MainWindow::on_actionWake_On_LAN_Magic_Packet_triggered()
-{
-    static QString previousMAC = "";
-    static int previousPortIndex = 0;
-    WakeOnLAN * wol = new WakeOnLAN(this);
-    wol->setTarget(previousMAC, previousPortIndex);
-    wol->setModal(true);
-    wol->exec();
-    if(!wol->generatedPacket.errorString.isEmpty()) {
-
-        QMessageBox msgBox;
-        msgBox.setWindowIcon(QIcon(":pslogo.png"));
-        msgBox.setWindowTitle(tr("Invalid MAC Address."));
-        msgBox.setStandardButtons(QMessageBox::Ok);
-        msgBox.setDefaultButton(QMessageBox::Ok);
-        msgBox.setIcon(QMessageBox::Critical);
-        msgBox.setText(wol->generatedPacket.errorString);
-        msgBox.setModal(true);
-        msgBox.exec();
-
-        on_actionWake_On_LAN_Magic_Packet_triggered();
-        wol->deleteLater();
-        return;
-    }
-
-    if(wol->generatedPacket.toIP.isEmpty()) {
-        wol->deleteLater();
-        return;
-    }
-
-    previousMAC = wol->mac;
-    previousPortIndex = wol->portIndex;
-
-    ui->packetIPEdit->setText(wol->generatedPacket.toIP);
-    ui->packetPortEdit->setText(QString::number(wol->generatedPacket.port));
-    ui->resendEdit->setText("0");
-    int findtext = ui->udptcpComboBox->findText(wol->generatedPacket.tcpOrUdp);
-    if (findtext > -1) {
-        ui->udptcpComboBox->setCurrentIndex(findtext);
-    }
-    ui->packetHexEdit->setText(wol->generatedPacket.hexString);
-    on_packetHexEdit_editingFinished();
-    on_udptcpComboBox_currentIndexChanged("");
-
-    if(ui->packetNameEdit->text().isEmpty()) {
-        ui->packetNameEdit->setFocus();
-    }
-    wol->deleteLater();
 }
 
