@@ -11,6 +11,7 @@
 #include "packetnetwork.h"
 #include "globals.h"
 #include "settings.h"
+#include<qstring.h>
 #include <QCoreApplication>
 #include <QFile>
 #include <QFileInfo>
@@ -21,6 +22,15 @@
 #include <QtGlobal>
 #include <QUrlQuery>
 #include <QStandardPaths>
+#include <iostream>
+#include "association.h"
+#include <QStringList>
+#include "dtlsthread.h"
+#include "dtlsserver.h"
+
+
+
+
 #ifdef CONSOLE_BUILD
 class QMessageBox {
 public:
@@ -145,6 +155,39 @@ QString PacketNetwork::getIPmode()
     return ipMode;
 }
 
+
+bool PacketNetwork::DTLSisSupported()
+{
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+
+    return false;
+#endif
+
+    if (!QSslSocket::supportsSsl()) {
+        return false;
+    }
+
+    // For now, only openssl is supported.
+    QString sslVersion = QSslSocket::sslLibraryBuildVersionString();
+    return sslVersion.toLower().contains("openssl");
+
+}
+
+
+bool PacketNetwork::DTLSListening()
+{
+    QUdpSocket * dtls;
+    QDEBUGVAR(dtlsServers.size());
+    foreach(dtls, dtlsServers) {
+        QDEBUGVAR(dtls->state());
+        if(dtls->state() == QAbstractSocket::BoundState) {
+            return true;
+        }
+    }
+    return false;
+}
+
 bool PacketNetwork::UDPListening()
 {
     QUdpSocket * udp;
@@ -152,7 +195,6 @@ bool PacketNetwork::UDPListening()
     foreach(udp, udpServers) {
         QDEBUGVAR(udp->state());
         if(udp->state() == QAbstractSocket::BoundState) {
-        //if(udp->state() == QAbstractSocket::ConnectedState) {
             return true;
         }
     }
@@ -219,7 +261,7 @@ void PacketNetwork::init()
 
     static bool erroronce = false;
 
-
+    dtlsServers.clear();
     tcpServers.clear();
     udpServers.clear();
     sslServers.clear();
@@ -231,15 +273,38 @@ void PacketNetwork::init()
 
     QSettings settings(SETTINGSFILE, QSettings::IniFormat);
 
-    QList<int> udpPortList, tcpPortList, sslPortList;
+    QList<int> udpPortList, tcpPortList, sslPortList, dtlsPortList;
+    int dtlsPort = 0;
     int udpPort = 0;
     int tcpPort = 0;
     int sslPort = 0;
 
+    dtlsPortList = Settings::portsToIntList(settings.value("dtlsPort", "0").toString());
     udpPortList = Settings::portsToIntList(settings.value("udpPort", "0").toString());
     tcpPortList = Settings::portsToIntList(settings.value("tcpPort", "0").toString());
     sslPortList = Settings::portsToIntList(settings.value("sslPort", "0").toString());
 
+    if(settings.value("dtlsServerEnable", "").toString().isEmpty()) {
+        settings.setValue("dtlsServerEnable", false);
+        settings.sync();
+    }
+    if(settings.value("udpServerEnable", "").toString().isEmpty()) {
+        settings.setValue("udpServerEnable", true);
+    }
+    if(settings.value("tcpServerEnable", "").toString().isEmpty()) {
+        settings.setValue("tcpServerEnable", false);
+    }
+    if(settings.value("sslServerEnable", "").toString().isEmpty()) {
+        settings.setValue("sslServerEnable", false);
+        settings.sync();
+    }
+
+    activateDTLS = settings.value("dtlsServerEnable", false).toBool();
+    activateUDP = settings.value("udpServerEnable", true).toBool();
+    activateTCP = settings.value("tcpServerEnable", false).toBool();
+    activateSSL = settings.value("sslServerEnable", false).toBool();
+
+
     QString ipMode = settings.value("ipMode", "0.0.0.0").toString();
     QDEBUGVAR(ipMode);
     QMessageBox msgBoxBindError;
@@ -253,14 +318,62 @@ void PacketNetwork::init()
 #ifdef RENDER_ONLY
     tcpPortList.clear();
     sslPortList.clear();
+    dtlsPortList.clear();
 #endif
 
 
-    QUdpSocket *udpSocket;
+    QUdpSocket *udpSocket, *dtlsSocket;
     ThreadedTCPServer *ssl, *tcp;
 
+
+
+
+    foreach (dtlsPort, dtlsPortList) {
+
+        if(!activateDTLS) {
+            continue;
+        }
+        bool bindResult = dtlsServer.listen(IPV4_OR_IPV6, dtlsPort);
+
+
+        dtlsSocket = &(dtlsServer.serverSocket);
+
+        dtlsSocket->setSocketOption(QAbstractSocket::MulticastTtlOption, 128);
+
+        if ((!bindResult) && (!erroronce)) {
+            QDEBUGVAR(dtlsPort);
+            erroronce = true;
+            if ((dtlsPort < 1024) && (dtlsPort > 0)) {
+                QString msgText = lowPortText;
+                msgText.replace("[PORT]", QString::number(dtlsPort));
+                msgBoxBindError.setText(msgText);
+                msgBoxBindError.exec();
+
+            } else {
+                QString msgText = portConsumedText;
+                msgText.replace("[PORT]", QString::number(dtlsPort));
+                msgBoxBindError.setText(msgText);
+                msgBoxBindError.exec();
+
+            }
+            dtlsSocket->close();
+            dtlsSocket->deleteLater();
+
+        }
+
+        if(bindResult) {
+            dtlsServers.append(dtlsSocket);
+        }
+
+    }
+
+
     foreach (udpPort, udpPortList) {
 
+        if(!activateUDP) {
+            continue;
+        }
+
 
         udpSocket = new QUdpSocket(this);
 
@@ -273,7 +386,7 @@ void PacketNetwork::init()
         if ((!bindResult) && (!erroronce)) {
             QDEBUGVAR(udpPort);
             erroronce = true;
-            if (udpPort < 1024 && udpPort > 0) {
+            if ((udpPort < 1024) && (udpPort > 0)) {
                 QString msgText = lowPortText;
                 msgText.replace("[PORT]", QString::number(udpPort));
                 msgBoxBindError.setText(msgText);
@@ -301,6 +414,11 @@ void PacketNetwork::init()
 
     foreach (tcpPort, tcpPortList) {
 
+        if(!activateTCP) {
+            continue;
+        }
+
+
         tcp = new ThreadedTCPServer(this);
         tcp->init(tcpPort, false, ipMode);
         tcpServers.append(tcp);
@@ -309,6 +427,10 @@ void PacketNetwork::init()
 
     foreach (sslPort, sslPortList) {
 
+        if(!activateSSL) {
+            continue;
+        }
+
         ssl = new ThreadedTCPServer(this);
         ssl->init(sslPort, true, ipMode);
         sslServers.append(ssl);
@@ -348,9 +470,6 @@ void PacketNetwork::init()
 
     sendResponse = settings.value("sendReponse", false).toBool();
     responseData = (settings.value("responseHex", "")).toString();
-    activateUDP = settings.value("udpServerEnable", true).toBool();
-    activateTCP = settings.value("tcpServerEnable", true).toBool();
-    activateSSL = settings.value("sslServerEnable", true).toBool();
     receiveBeforeSend = settings.value("attemptReceiveCheck", false).toBool();
     persistentConnectCheck = settings.value("persistentConnectCheck", false).toBool();
     sendSmartResponse = settings.value("smartResponseEnableCheck", false).toBool();
@@ -367,18 +486,38 @@ void PacketNetwork::init()
 #ifdef RENDER_ONLY
     activateTCP = false;
     activateSSL = false;
+    activateDTLS = false;
     smartList.clear();
 #endif
     if (settings.value("delayAfterConnectCheck", false).toBool()) {
         delayAfterConnect = 500;
     }
 
+    if (activateDTLS) {
+        foreach (dtlsSocket, dtlsServers) {
+            connect(&dtlsServer, SIGNAL(serverPacketReceived(Packet)), this, SLOT(packetReceivedECHO(Packet)),Qt::UniqueConnection);
+            connect(&dtlsServer, SIGNAL(serverPacketSent(Packet)), this, SLOT(packetSentECHO(Packet)),Qt::UniqueConnection);
+        }
+
+    } else {
+        QDEBUG() << "udp server disable";
+        foreach (dtlsSocket, dtlsServers) {
+            dtlsSocket->close();
+        }
+        dtlsServers.clear();
+
+    }
+
+
+
     if (activateUDP) {
+
         foreach (udpSocket, udpServers) {
             QDEBUG() << "signal/slot datagram connect: " << connect(udpSocket, SIGNAL(readyRead()),
                      this, SLOT(readPendingDatagrams()));
         }
 
+
     } else {
         QDEBUG() << "udp server disable";
         foreach (udpSocket, udpServers) {
@@ -415,6 +554,27 @@ void PacketNetwork::init()
 
 //TODO add timed event feature?
 
+QList<int> PacketNetwork::getDTLSPortsBound()
+{
+    QList<int> pList;
+    pList.clear();
+    QUdpSocket * dtlsServer;
+    foreach (dtlsServer, dtlsServers) {
+        if(dtlsServer->BoundState == QAbstractSocket::BoundState) {
+            if(dtlsServer){
+                pList.append(dtlsServer->localPort());
+            }
+        }
+    }
+    return pList;
+
+}
+
+QString PacketNetwork::getDTLSPortString()
+{
+
+    return Settings::intListToPorts(getDTLSPortsBound());
+}
 
 QList<int> PacketNetwork::getUDPPortsBound()
 {
@@ -601,8 +761,7 @@ void PacketNetwork::readPendingDatagrams()
             sender =  theDatagram.senderAddress();
             senderPort = theDatagram.senderPort();
 
-            QDEBUG() << "data size is" << datagram.size() << "Sender is" << sender << ":" << senderPort;
-    //        QDEBUG() << debugQByteArray(datagram);
+            QDEBUG() << "data size is" << datagram.size();
 
             Packet udpPacket;
             udpPacket.timestamp = QDateTime::currentDateTime();
@@ -677,6 +836,7 @@ void PacketNetwork::readPendingDatagrams()
 }
 
 
+
 QString PacketNetwork::debugQByteArray(QByteArray debugArray)
 {
     QString outString = "";
@@ -822,6 +982,58 @@ void PacketNetwork::packetToSend(Packet sendpacket)
     sendpacket.timestamp = QDateTime::currentDateTime();
     sendpacket.name = sendpacket.timestamp.toString(DATETIMEFORMAT);
 
+
+    if(sendpacket.isDTLS()){
+        Dtlsthread * thread = new Dtlsthread(sendpacket, this);
+        QSettings settings(SETTINGSFILE, QSettings::IniFormat);
+        QDEBUG() << connect(thread, SIGNAL(packetReceived(Packet)), this, SLOT(packetReceivedECHO(Packet)))
+                 << connect(thread, SIGNAL(toStatusBar(QString, int, bool)), this, SLOT(toStatusBarECHO(QString, int, bool)))
+                 << connect(thread, SIGNAL(packetSent(Packet)), this, SLOT(packetSentECHO(Packet)))
+                 << connect(thread, SIGNAL(destroyed()), this, SLOT(disconnected()));
+
+        bool leaveSessionOpen = false;
+#ifndef CONSOLE_BUILD
+        leaveSessionOpen = settings.value("leaveSessionOpen").toString() == "true";
+#endif
+        if(leaveSessionOpen){
+#ifndef CONSOLE_BUILD
+
+            thread->persistentRequest=true;
+            PersistentConnection * pcWindow = new PersistentConnection();
+
+            pcWindow->sendPacket = sendpacket;
+            pcWindow->dthread = thread;
+            pcWindow->init();
+
+
+            QDEBUG() << ": thread Connection attempt "
+                     << connect(pcWindow, SIGNAL(persistentPacketSend(Packet)), thread, SLOT(sendPersistant(Packet)))
+                     << connect(pcWindow, SIGNAL(closeConnection()), thread, SLOT(closeConnection()))
+                     << connect(thread, SIGNAL(connectStatus(QString)), pcWindow, SLOT(statusReceiver(QString)))
+                     << connect(thread, SIGNAL(packetSent(Packet)), pcWindow, SLOT(packetSentSlot(Packet)));
+
+            pcWindow->show();
+            thread->start();
+
+#endif
+
+        }
+        else{
+            //connect(*(thread->dtlsAssociation), &QDtls::handshakeTimeout, *thread, &Dtlsthread::onHandshakeTimeout);
+
+            QTimer* timer = new QTimer(this);
+            //timer->setSingleShot(true); // Make the timer single-shot
+            thread->timer = timer;
+            connect(timer, SIGNAL(timeout()), thread, SLOT(onTimeout()));
+            timer->start(1000);
+            thread->start();
+
+        }
+
+        dtlsthreadList.append(thread);
+    }
+
+
     if (sendpacket.isUDP()) {
         QUdpSocket * sendUDP;
         bool oneoff = false;
@@ -940,10 +1152,9 @@ void PacketNetwork::packetToSend(Packet sendpacket)
 
     }
 
-
-
 }
 
+
 void PacketNetwork::httpError(QNetworkRequest* pReply)
 {
     QDEBUGVAR(pReply);
@@ -1062,3 +1273,58 @@ QList<ThreadedTCPServer *> PacketNetwork::allTCPServers()
     return theServers;
 }
 
+std::vector<QString> PacketNetwork::getCmdInput(Packet sendpacket, QSettings& settings){
+    //the array of cmdComponents: dataStr, toIp, toPort, sslPrivateKeyPath, sslLocalCertificatePath, sslCaFullPath
+    std::vector<QString> cmdComponents;
+
+    //get the data of the packet
+    cmdComponents.push_back(QString::fromUtf8(sendpacket.getByteArray()));
+    cmdComponents.push_back(sendpacket.toIP);
+    cmdComponents.push_back(QString::number(sendpacket.port));
+
+    //get the pathes for verification from the settings
+    cmdComponents.push_back(settings.value("sslPrivateKeyPath", "default").toString());
+    cmdComponents.push_back(settings.value("sslLocalCertificatePath", "default").toString());
+    QString sslCaPath = settings.value("sslCaPath", "default").toString();
+
+    //get the full path to to ca-signed-cert.pem file
+    QDir dir(sslCaPath);
+    if (dir.exists()) {
+        QStringList nameFilters;
+        nameFilters << "*.pem";  // Filter for .txt files
+
+        dir.setNameFilters(nameFilters);
+        QStringList fileList = dir.entryList();
+
+        if (!fileList.isEmpty()) {
+            // Select the first file that matches the filter
+            cmdComponents.push_back(dir.filePath(fileList.first()));
+        } else {
+            qDebug() << "No matching files found.";
+        }
+    } else {
+        qDebug() << "Directory does not exist.";
+    }
+    cmdComponents.push_back(settings.value("cipher", "AES256-GCM-SHA384").toString());
+    return cmdComponents;
+}
+
+
+
+
+void PacketNetwork::on_twoVerify_StateChanged(){
+    QSettings settings(SETTINGSFILE, QSettings::IniFormat);
+    QString twoVerify = settings.value("twoVerify", "false").toString();
+    if(twoVerify == "false"){
+        settings.setValue("twoVerify", "true");
+        dtlsServer.serverConfiguration.setPeerVerifyMode(QSslSocket::VerifyPeer);
+    }
+    else{
+        settings.setValue("twoVerify", "false");
+        dtlsServer.serverConfiguration.setPeerVerifyMode(QSslSocket::VerifyNone);
+
+    }
+}
+
+
+
