@@ -30,6 +30,8 @@ class TCPThread : public QThread
         bool closeRequest;
         bool isSecure;
         bool isEncrypted();
+        Packet packetReply;
+        bool consoleMode;
 
     signals:
         void error(QSslSocket::SocketError socketError);
