@@ -17,6 +17,9 @@ class ThreadedTCPServer : public QTcpServer
         bool encrypted;
 
         bool init(quint16 port, bool isEncrypted, QString ipMode);
+        bool consoleMode;
+        void responsePacket(Packet packetToSend);
+
     protected:
         void incomingConnection(qintptr socketDescriptor);
 
@@ -30,11 +33,14 @@ class ThreadedTCPServer : public QTcpServer
         void packetReceivedECHO(Packet sendpacket);
         void toStatusBarECHO(const QString & message, int timeout = 0, bool override = false);
         void packetSentECHO(Packet sendpacket);
+        void outputTCPPacket(Packet receivePacket);
 
 
 
     private:
         QList<TCPThread *> threads;
+        Packet packetReply;
+
 
 
         QList<TCPThread *> tcpthreadList;
